@@ -2,6 +2,7 @@ package runner
 
 import (
 	"github.com/projectdiscovery/gologger"
+	"github.com/projectdiscovery/utils/auth/pdcp"
 	updateutils "github.com/projectdiscovery/utils/update"
 )
 
@@ -12,11 +13,12 @@ const banner = `
  | (_| || | | |\__ \ /  \ 
   \__,_||_| |_||___//_/\_\
 `
+
 // Name
 const ToolName = `dnsx`
 
 // version is the current version of dnsx
-const version = `1.1.4`
+const version = `1.2.3`
 
 // showBanner is used to show the banner to the user
 func showBanner() {
@@ -30,4 +32,10 @@ func GetUpdateCallback() func() {
 		showBanner()
 		updateutils.GetUpdateToolCallback("dnsx", version)()
 	}
-}
\ No newline at end of file
+}
+
+// AuthWithPDCP is used to authenticate with PDCP
+func AuthWithPDCP() {
+	showBanner()
+	pdcp.CheckNValidateCredentials("dnsx")
+}
