@@ -3,6 +3,7 @@ package runner
 import (
 	"bufio"
 	"context"
+	"encoding/json"
 	"fmt"
 	"io"
 	"os"
@@ -10,6 +11,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/logrusorgru/aurora"
 	"github.com/miekg/dns"
 	"github.com/pkg/errors"
 	asnmap "github.com/projectdiscovery/asnmap/libs"
@@ -24,6 +26,8 @@ import (
 	"github.com/projectdiscovery/retryabledns"
 	fileutil "github.com/projectdiscovery/utils/file"
 	iputil "github.com/projectdiscovery/utils/ip"
+	mapsutil "github.com/projectdiscovery/utils/maps"
+	sliceutil "github.com/projectdiscovery/utils/slice"
 )
 
 // Runner is a client for running the enumeration process.
@@ -36,14 +40,14 @@ type Runner struct {
 	workerchan          chan string
 	outputchan          chan string
 	wildcardworkerchan  chan string
-	wildcards           map[string]struct{}
-	wildcardsmutex      sync.RWMutex
+	wildcards           *mapsutil.SyncLockMap[string, struct{}]
 	wildcardscache      map[string][]string
 	wildcardscachemutex sync.Mutex
 	limiter             *ratelimit.Limiter
 	hm                  *hybrid.HybridMap
 	stats               clistats.StatisticsClient
 	tmpStdinFile        string
+	aurora              aurora.Aurora
 }
 
 func New(options *Options) (*Runner, error) {
@@ -54,6 +58,8 @@ func New(options *Options) (*Runner, error) {
 	dnsxOptions.TraceMaxRecursion = options.TraceMaxRecursion
 	dnsxOptions.Hostsfile = options.HostsFile
 	dnsxOptions.OutputCDN = options.OutputCDN
+	dnsxOptions.Proxy = options.Proxy
+	dnsxOptions.Timeout = options.Timeout
 	if options.Resolvers != "" {
 		dnsxOptions.BaseResolvers = []string{}
 		// If it's a file load resolvers from it
@@ -89,6 +95,9 @@ func New(options *Options) (*Runner, error) {
 	if options.SOA {
 		questionTypes = append(questionTypes, dns.TypeSOA)
 	}
+	if options.ANY {
+		questionTypes = append(questionTypes, dns.TypeANY)
+	}
 	if options.TXT {
 		questionTypes = append(questionTypes, dns.TypeTXT)
 	}
@@ -111,6 +120,7 @@ func New(options *Options) (*Runner, error) {
 		questionTypes = append(questionTypes, dns.TypeA)
 	}
 	dnsxOptions.QuestionTypes = questionTypes
+	dnsxOptions.QueryAll = options.QueryAll
 
 	dnsX, err := dnsx.New(dnsxOptions)
 	if err != nil {
@@ -135,6 +145,10 @@ func New(options *Options) (*Runner, error) {
 		}
 	}
 
+	if os.Getenv("NO_COLOR") == "true" {
+		options.NoColor = true
+	}
+
 	r := Runner{
 		options:            options,
 		dnsx:               dnsX,
@@ -143,11 +157,12 @@ func New(options *Options) (*Runner, error) {
 		wgwildcardworker:   &sync.WaitGroup{},
 		workerchan:         make(chan string),
 		wildcardworkerchan: make(chan string),
-		wildcards:          make(map[string]struct{}),
+		wildcards:          mapsutil.NewSyncLockMap[string, struct{}](),
 		wildcardscache:     make(map[string][]string),
 		limiter:            limiter,
 		hm:                 hm,
 		stats:              stats,
+		aurora:             aurora.NewAurora(!options.NoColor),
 	}
 
 	return &r, nil
@@ -157,7 +172,12 @@ func (r *Runner) InputWorkerStream() {
 	var sc *bufio.Scanner
 	// attempt to load list from file
 	if fileutil.FileExists(r.options.Hosts) {
-		f, _ := os.Open(r.options.Hosts)
+		f, err := os.Open(r.options.Hosts)
+		if err != nil {
+			gologger.Error().Msgf("Could not open hosts file '%s': %s", r.options.Hosts, err)
+			return
+		}
+		defer f.Close()
 		sc = bufio.NewScanner(f)
 	} else if fileutil.HasStdin() {
 		sc = bufio.NewScanner(os.Stdin)
@@ -226,8 +246,10 @@ func (r *Runner) prepareInput() error {
 			return err
 		}
 		// closes the file as we will read it multiple times to build the iterations
-		stdinFile.Close()
-		defer os.RemoveAll(r.tmpStdinFile)
+		_ = stdinFile.Close()
+		defer func() {
+			_ = os.RemoveAll(r.tmpStdinFile)
+		}()
 	}
 
 	if r.options.Domains != "" {
@@ -305,8 +327,15 @@ func (r *Runner) prepareInput() error {
 		r.stats.AddStatic("startedAt", time.Now())
 		r.stats.AddCounter("requests", 0)
 		r.stats.AddCounter("total", uint64(numHosts*len(r.dnsx.Options.QuestionTypes)))
+		r.stats.AddDynamic("summary", makePrintCallback())
 		// nolint:errcheck
-		r.stats.Start(makePrintCallback(), time.Duration(5)*time.Second)
+		r.stats.Start()
+		r.stats.GetStatResponse(time.Second*5, func(s string, err error) error {
+			if err != nil && r.options.Verbose {
+				gologger.Error().Msgf("Could not read statistics: %s\n", err)
+			}
+			return nil
+		})
 	}
 	return nil
 }
@@ -360,9 +389,9 @@ func normalize(data string) string {
 }
 
 // nolint:deadcode
-func makePrintCallback() func(stats clistats.StatisticsClient) {
+func makePrintCallback() func(stats clistats.StatisticsClient) interface{} {
 	builder := &strings.Builder{}
-	return func(stats clistats.StatisticsClient) {
+	return func(stats clistats.StatisticsClient) interface{} {
 		builder.WriteRune('[')
 		startedAt, _ := stats.GetStatic("startedAt")
 		duration := time.Since(startedAt.(time.Time))
@@ -392,7 +421,9 @@ func makePrintCallback() func(stats clistats.StatisticsClient) {
 		builder.WriteRune('\n')
 
 		fmt.Fprintf(os.Stderr, "%s", builder.String())
+		statString := builder.String()
 		builder.Reset()
+		return statString
 	}
 }
 
@@ -443,8 +474,7 @@ func (r *Runner) run() error {
 		// prepare in memory structure similarly to shuffledns
 		r.hm.Scan(func(k, v []byte) error {
 			var dnsdata retryabledns.DNSData
-			err := dnsdata.Unmarshal(v)
-			if err != nil {
+			if err := json.Unmarshal(v, &dnsdata); err != nil {
 				// the item has no record - ignore
 				return nil
 			}
@@ -461,6 +491,7 @@ func (r *Runner) run() error {
 			return nil
 		})
 
+		gologger.Debug().Msgf("Found %d unique IPs:%s\n", len(listIPs), strings.Join(listIPs, ", "))
 		// wildcard workers
 		numThreads := r.options.Threads
 		if numThreads > len(listIPs) {
@@ -498,7 +529,7 @@ func (r *Runner) run() error {
 						seen[host] = struct{}{}
 						_ = r.lookupAndOutput(host)
 					}
-				} else if _, ok := r.wildcards[host]; !ok {
+				} else if !r.wildcards.Has(host) {
 					if _, ok := seen[host]; !ok {
 						seen[host] = struct{}{}
 						_ = r.lookupAndOutput(host)
@@ -566,9 +597,13 @@ func (r *Runner) HandleOutput() {
 		if err != nil {
 			gologger.Fatal().Msgf("%s\n", err)
 		}
-		defer foutput.Close()
+		defer func() {
+			_ = foutput.Close()
+		}()
 		w = bufio.NewWriter(foutput)
-		defer w.Flush()
+		defer func() {
+			_ = w.Flush()
+		}()
 	}
 	for item := range r.outputchan {
 		if foutput != nil {
@@ -697,12 +732,23 @@ func (r *Runner) worker() {
 		}
 		// if wildcard filtering just store the data
 		if r.options.WildcardDomain != "" {
-			// nolint:errcheck
-			r.storeDNSData(dnsData.DNSData)
+			if err := r.storeDNSData(dnsData.DNSData); err != nil {
+				gologger.Debug().Msgf("Failed to store DNS data for %s: %v\n", domain, err)
+			}
 			continue
 		}
+
+		// if response type filter is set, we don't want to ignore them
+		if len(r.options.responseTypeFilterMap) > 0 && r.shouldSkipRecord(&dnsData) {
+			continue
+		}
+
 		if r.options.JSON {
-			jsons, _ := dnsData.JSON()
+			var marshalOptions []dnsx.MarshalOption
+			if r.options.OmitRaw {
+				marshalOptions = append(marshalOptions, dnsx.WithoutAllRecords())
+			}
+			jsons, _ := dnsData.JSON(marshalOptions...)
 			r.outputchan <- jsons
 			continue
 		}
@@ -710,45 +756,77 @@ func (r *Runner) worker() {
 			r.outputchan <- dnsData.Raw
 			continue
 		}
+
+		// if response type filter is set, then print filtered records, moved to below from above block
+		// coz json and raw flag support
+		if len(r.options.responseTypeFilterMap) > 0 {
+			r.outputRecordType(domain, dnsData.A, "A", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.AAAA, "AAAA", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.CNAME, "CNAME", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.MX, "MX", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.NS, "NS", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, sliceutil.Dedupe(dnsData.GetSOARecords()), "SOA", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.TXT, "TXT", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.SRV, "SRV", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.CAA, "CAA", dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.PTR, "PTR", dnsData.CDNName, dnsData.ASN)
+			continue
+		}
+
 		if r.options.hasRCodes {
 			r.outputResponseCode(domain, dnsData.StatusCodeRaw)
 			continue
 		}
+
 		if r.options.A {
-			r.outputRecordType(domain, dnsData.A, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.A, "A", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.AAAA {
-			r.outputRecordType(domain, dnsData.AAAA, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.AAAA, "AAAA", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.CNAME {
-			// fmt.Println("inside cname", dnsData.ASN)
-			r.outputRecordType(domain, dnsData.CNAME, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.CNAME, "CNAME", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.PTR {
-			r.outputRecordType(domain, dnsData.PTR, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.PTR, "PTR", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.MX {
-			r.outputRecordType(domain, dnsData.MX, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.MX, "MX", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.NS {
-			r.outputRecordType(domain, dnsData.NS, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.NS, "NS", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.SOA {
-			r.outputRecordType(domain, dnsData.SOA, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, sliceutil.Dedupe(dnsData.GetSOARecords()), "SOA", dnsData.CDNName, dnsData.ASN)
+		}
+		if r.options.ANY {
+			allParsedRecords := sliceutil.Merge(
+				dnsData.A,
+				dnsData.AAAA,
+				dnsData.CNAME,
+				dnsData.MX,
+				dnsData.PTR,
+				sliceutil.Dedupe(dnsData.GetSOARecords()),
+				dnsData.NS,
+				dnsData.TXT,
+				dnsData.SRV,
+				dnsData.CAA,
+			)
+			r.outputRecordType(domain, allParsedRecords, "ANY", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.TXT {
-			r.outputRecordType(domain, dnsData.TXT, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.TXT, "TXT", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.SRV {
-			r.outputRecordType(domain, dnsData.SRV, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.SRV, "SRV", dnsData.CDNName, dnsData.ASN)
 		}
 		if r.options.CAA {
-			r.outputRecordType(domain, dnsData.CAA, dnsData.CDNName, dnsData.ASN)
+			r.outputRecordType(domain, dnsData.CAA, "CAA", dnsData.CDNName, dnsData.ASN)
 		}
 	}
 }
 
-func (r *Runner) outputRecordType(domain string, items []string, cdnName string, asn *dnsx.AsnResponse) {
+func (r *Runner) outputRecordType(domain string, items interface{}, queryType, cdnName string, asn *dnsx.AsnResponse) {
 	var details string
 	if cdnName != "" {
 		details = fmt.Sprintf(" [%s]", cdnName)
@@ -756,12 +834,23 @@ func (r *Runner) outputRecordType(domain string, items []string, cdnName string,
 	if asn != nil {
 		details = fmt.Sprintf("%s %s", details, asn.String())
 	}
-	for _, item := range items {
+	var records []string
+
+	switch items := items.(type) {
+	case []string:
+		records = items
+	case []retryabledns.SOA:
+		for _, item := range items {
+			records = append(records, item.NS, item.Mbox)
+		}
+	}
+
+	for _, item := range records {
 		item := strings.ToLower(item)
 		if r.options.ResponseOnly {
 			r.outputchan <- fmt.Sprintf("%s%s", item, details)
 		} else if r.options.Response {
-			r.outputchan <- fmt.Sprintf("%s [%s] %s", domain, item, details)
+			r.outputchan <- fmt.Sprintf("%s [%s] [%s] %s", domain, r.aurora.Magenta(queryType), r.aurora.Green(item).String(), details)
 		} else {
 			// just prints out the domain if it has a record type and exit
 			r.outputchan <- fmt.Sprintf("%s%s", domain, details)
@@ -777,17 +866,69 @@ func (r *Runner) outputResponseCode(domain string, responsecode int) {
 	}
 }
 
+func (r *Runner) shouldSkipRecord(dnsData *dnsx.ResponseData) bool {
+	for _, et := range r.options.responseTypeFilterMap {
+		switch strings.ToLower(strings.TrimSpace(et)) {
+		case "a":
+			if len(dnsData.A) > 0 {
+				return true
+			}
+		case "aaaa":
+			if len(dnsData.AAAA) > 0 {
+				return true
+			}
+		case "cname":
+			if len(dnsData.CNAME) > 0 {
+				return true
+			}
+		case "ns":
+			if len(dnsData.NS) > 0 {
+				return true
+			}
+		case "txt":
+			if len(dnsData.TXT) > 0 {
+				return true
+			}
+		case "mx":
+			if len(dnsData.MX) > 0 {
+				return true
+			}
+		case "soa":
+			if len(dnsData.SOA) > 0 {
+				return true
+			}
+		case "srv":
+			if len(dnsData.SRV) > 0 {
+				return true
+			}
+		case "ptr":
+			if len(dnsData.PTR) > 0 {
+				return true
+			}
+		case "caa":
+			if len(dnsData.CAA) > 0 {
+				return true
+			}
+		default:
+			return false
+		}
+	}
+	return false
+}
+
 func (r *Runner) storeDNSData(dnsdata *retryabledns.DNSData) error {
-	data, err := dnsdata.Marshal()
+	dnsdata.RawResp = nil
+
+	data, err := dnsdata.JSON()
 	if err != nil {
 		return err
 	}
-	return r.hm.Set(dnsdata.Host, data)
+	return r.hm.Set(dnsdata.Host, []byte(data))
 }
 
 // Close running instance
 func (r *Runner) Close() {
-	r.hm.Close()
+	_ = r.hm.Close()
 }
 
 func (r *Runner) wildcardWorker() {
@@ -798,12 +939,9 @@ func (r *Runner) wildcardWorker() {
 		if !more {
 			break
 		}
-
 		if r.IsWildcard(host) {
 			// mark this host as a wildcard subdomain
-			r.wildcardsmutex.Lock()
-			r.wildcards[host] = struct{}{}
-			r.wildcardsmutex.Unlock()
+			_ = r.wildcards.Set(host, struct{}{})
 		}
 	}
 }
